(function() {
var GridViewFilterHelper = ASPx.CreateClass(ASPx.GridFilterHelper, {
 constructor: function(grid) {
  this.constructor.prototype.constructor.call(this, grid);
  this.initialFilterRowState = { };
  this.filterKeyPressTimerId = -1;
  this.filterKeyPressEditor = null;
  this.preparedToCallbackFilterValue = undefined;
  this.preparedFilterValueClearDelay = 150;
  this.filterRowMenuColumnIndex = -1;
  this.currentCheckedItemIndex = -1;
 },
 GetCustDialogHelper: function() { return this.grid.GetCustDialogHelper(); },
 GetBatchEditHelper: function() { return this.grid.GetBatchEditHelper(); },
 GetAutoFilterDelay: function() { return this.grid.autoFilterDelay; },
 IsMultiColumnAutoFilter: function() { return this.grid.allowMultiColumnAutoFilter; },
 GetFilterRowMenu: function() { return this.grid.GetFilterRowMenu(); },
 GetAutoFilterEditor: function(columnIndex) { return this.grid.GetAutoFilterEditor(columnIndex); },
 GetFilterConditionState: function() { return this.grid.filterRowConditions; },
 AutoFilterByColumnCore: function(columnIndex, editorValue) { 
  this.grid.AutoFilterByColumn(columnIndex, editorValue); 
 },
 AutoFilterByMultipleColumns: function(args) {
  this.grid.gridCallBack([ASPxClientGridViewCallbackCommand.ApplyMultiColumnFilter].concat(args));
 },
 SendCallbackToChangeFilterCondition: function(args) {
  this.grid.gridCallBack([ASPxClientGridViewCallbackCommand.FilterRowMenu].concat(args));
 },
 GetColumnIndicesForFilterRow: function() { 
  var columnHelper = this.grid.GetColumnHelper();
  if(this.GetCustDialogHelper())
   return columnHelper.GetAllColumns().map(function(c) { return c.index; });
  return columnHelper.GetVisibleColumnIndices();
 },
 GetFilterCondition: function(columnIndex) { 
  var state = this.GetFilterConditionState();
  return state ? state[columnIndex] : "";
 },
 SetFilterCondition: function(columnIndex, value) { 
  var state = this.GetFilterConditionState();
  if(state)
   state[columnIndex] = value; 
 },
 IsCustDialogVisible: function() {
  var helper = this.GetCustDialogHelper();
  return helper && helper.IsDialogVisible();
 },
 AutoFilterByColumn: function(columnIndex, editorValue) { 
  if(!this.IsCustDialogVisible())
   this.AutoFilterByColumnCore(columnIndex, editorValue); 
 },
 IsBatchUpdateConfirmLocked: function() {
  var batchHelper = this.GetBatchEditHelper();
  return batchHelper && batchHelper.IsUpdateConfirmLocked();
 },
 ApplyColumnAutoFilterCore: function(editor) {
  if(this.IsFilterLocked()) 
   return;
  this.ClearAutoFilterInputTimer();
  var editorValue = editor.GetValueString();
  if(this.IsBatchUpdateConfirmLocked() && this.preparedToCallbackFilterValue != "undefined"  && editorValue === this.preparedToCallbackFilterValue) 
   return;
  var columnIndex = this.ResolveColumnIndex(editor.name);
  if(columnIndex < 0 || !this.IsColumnAutoFilterChanged(columnIndex)) 
   return;
  this.SaveFilterEditorActiveElement(editor);
  this.preparedToCallbackFilterValue = editorValue;
  window.setTimeout(function() { delete this.preparedToCallbackFilterValue;  }.bind(this), this.preparedFilterValueClearDelay);
  this.AutoFilterByColumn(columnIndex, editorValue);
  this.OnChange(columnIndex);
 },
 ApplyMultiColumnAutoFilter: function(editor) {
  if(this.IsFilterLocked()) 
   return;
  this.SaveFilterEditorActiveElement(editor);
  var args = this.SerializeChangedAutoFilterValues();
  if(args.length > 0)
   this.AutoFilterByMultipleColumns(args);
  var name = editor ? editor.name : "";
  this.OnChange(this.ResolveColumnIndex(name));
 },
 SerializeChangedAutoFilterValues: function() {
  var result = [ ];
  var changedValues = this.GetChangedAutoFilterValues();
  for(var columnIndex in changedValues) {
   if(!changedValues.hasOwnProperty(columnIndex)) continue;
   result.push(columnIndex);
   result.push(changedValues[columnIndex].value);
   result.push(changedValues[columnIndex].filterCondition);
  }
  return result;
 },
 SaveFilterEditorsState: function() {
  ASPx.GridFilterHelper.prototype.SaveFilterEditorsState.call(this);
  this.initialFilterRowState = this.GetFilterRowState();
 },
 ClearFilterEditorsState: function() { 
  ASPx.GridFilterHelper.prototype.ClearFilterEditorsState.call(this);
  this.initialFilterRowState = { };
 },
 GetFilterRowState: function() {
  var indices = this.GetColumnIndicesForFilterRow();
  return ASPx.Data.ArrayToHash(indices, function(columnIndex) { return columnIndex; }, function(columnIndex) { 
   var editor = this.GetAutoFilterEditor(columnIndex);
   return {
    value: editor && editor.GetMainElement() ? editor.GetValueString() : "",
    filterCondition: this.GetFilterCondition(columnIndex)
   };
  }.bind(this));
 },
 GetChangedAutoFilterValues: function() {
  var result = { };
  var currentFilterRowState = this.GetFilterRowState();
  for(var columnIndex in this.initialFilterRowState) {
   if(!this.initialFilterRowState.hasOwnProperty(columnIndex)) continue;
   columnIndex = parseInt(columnIndex);
   var initialState = this.initialFilterRowState[columnIndex];
   var currentState = currentFilterRowState[columnIndex];
   if(currentState.value !== initialState.value || currentState.filterCondition !== initialState.filterCondition)
    result[columnIndex] = { filterCondition: currentState.filterCondition, value: currentState.value !== null ? currentState.value : "" };
  }
  return result;
 },
 OnColumnFilterInputKeyPress: function(editor, e) {
  if(this.OnColumnFilterInputSpecKeyPress(editor, e))
   return;
  this.ClearAutoFilterInputTimer();
  this.CreateAutoFilterTimer(editor);
  this.OnChange(this.ResolveColumnIndex(editor.name));
 },
 OnColumnFilterInputSpecKeyPress: function(editor, e) {
  if(e.htmlEvent) 
   e = e.htmlEvent;
  if(e.keyCode == ASPx.Key.Tab) 
   return true;
  if(e.keyCode == ASPx.Key.Enter) {
   ASPx.Evt.PreventEventAndBubble(e);
   window.setTimeout(function() {
    editor.Validate();
    if(this.IsMultiColumnAutoFilter())
     this.ApplyMultiColumnAutoFilter(editor);
    else
     this.ApplyColumnAutoFilterCore(editor);
   }.bind(this), 0);
   return true;
  }
  if(e.keyCode == ASPx.Key.Delete && e.ctrlKey) {
   ASPx.Evt.PreventEventAndBubble(e);
   window.setTimeout(function() {
    editor.SetValue(null);
    if(!this.IsMultiColumnAutoFilter())
     this.ApplyColumnAutoFilterCore(editor);
   }.bind(this), 0);
   return true;
  }
  return false;
 },
 OnFilterKeyPressTick: function() {
  if(this.filterKeyPressEditor && this.IsValidInstance())
   this.ApplyColumnAutoFilterCore(this.filterKeyPressEditor);
 },
 OnColumnFilterInputChanged: function(editor) {
  this.ApplyColumnAutoFilterCore(editor);
 },
 OnColumnFilterUserInput: function(editor) {
  if(this.IsAutoFilterTimerActive()) return;
  this.ColumnFilterOnPaste(editor);
 },
 ColumnFilterOnPaste: function(editor) {
  this.EnsureEditorRawValue(editor);
  this.CreateAutoFilterTimer(editor);
 },
 IsAutoFilterTimerActive: function() {
  return this.filterKeyPressTimerId !== -1;
 },
 CreateAutoFilterTimer: function(editor) {
  this.filterKeyPressEditor = editor;
  this.filterKeyPressTimerId = window.setTimeout(this.OnFilterKeyPressTick.bind(this), this.GetAutoFilterDelay());
 },
 ClearAutoFilterInputTimer: function() {
  this.filterKeyPressTimerId = ASPx.Timer.ClearTimer(this.filterKeyPressTimerId);
 },
 OnChange: function(columnIndex) {
  if(this.IsCustDialogVisible())
   window.setTimeout(function() { this.GetCustDialogHelper().ProcessFilterChange(columnIndex); }.bind(this), 0);
 },
 ClearColumnFilter: function(columnIndex) {
  this.ClearColumnAutoFilter(columnIndex);
  this.ClearColumnHeaderFilter(columnIndex);
  this.OnChange(columnIndex);
 },
 ClearColumnAutoFilter: function(columnIndex) {
  var editor = this.GetAutoFilterEditor(columnIndex);
  if(editor)
   editor.SetValue(null);   
 },
 ClearColumnHeaderFilter: function(columnIndex) {
  var column = this.GetColumn(columnIndex);
  var hfMode = column.HFMode;
  var listBox = this.GetListBox(columnIndex);
  if(listBox) {
   listBox.UnselectAll();
   if(hfMode ===  ASPx.GridHeaderFilterMode.List)
    listBox.SelectIndex(0);
  }
  if(hfMode === ASPx.GridHeaderFilterMode.CheckedList) {
   var checkbox = this.GetSelectAllCheckBox(columnIndex); 
   if(checkbox)
    checkbox.SetCheckState(ASPx.CheckBoxCheckState.Unchecked);
  }
  if(hfMode === ASPx.GridHeaderFilterMode.DateRangePicker) {
   var dateEdit_From = this.GetDateEdit_From(columnIndex);
   if(dateEdit_From) 
    dateEdit_From.SetValue(null);
   var dateEdit_To = this.GetDateEdit_To(columnIndex);
   if(dateEdit_To)
    dateEdit_To.SetValue(null);
  }
  if(hfMode === ASPx.GridHeaderFilterMode.DateRangeCalendar) {
   var calendar = this.GetCalendar(columnIndex);
   if(calendar)
    calendar.SetValue(null);
  }
 },
 HasFilter: function(columnIndex) {
  return this.HasHeaderFilter(columnIndex) || this.HasAutoFilter(columnIndex);
 },
 HasHeaderFilter: function(columnIndex) {
  var state = this.GetHeaderFilterState(columnIndex);
  var column = this.GetColumn(columnIndex);
  if(!state || !column)
   return false;
  if(column.HFMode === ASPx.GridHeaderFilterMode.List && state.listBoxSelectedIndices[0] === 0)
   return false;
  var hasDateRange = state.pickerRange && (state.pickerRange.start || state.pickerRange.end);
  var hasSelectedDates = state.calendarDates && state.calendarDates.length > 0;
  return state.listBoxSelectedIndices.length > 0 || hasDateRange || hasSelectedDates;  
 },
 HasAutoFilter: function(columnIndex) {
  var editor = this.GetAutoFilterEditor(columnIndex);   
  var value = editor && editor.GetMainElement() && editor.GetValueString();
  return !!value;
 },
 IsColumnAutoFilterChanged: function(columnIndex) {   
  var editor = this.GetAutoFilterEditor(columnIndex);
  var initState = this.initialFilterRowState[columnIndex];
  if (!editor || !editor.GetMainElement() || !initState) 
   return false;
  var currentValue = editor.GetValueString();
  return initState.value !== currentValue || initState.filterCondition !== this.GetFilterCondition(columnIndex);
 },
 FilterRowMenuButtonClick: function(columnIndex, element) {
  var menu = this.GetFilterRowMenu();
  if(!menu) return;
  var column = this.GetColumn(columnIndex);
  if(!column) return;
  var checkedItemIndex;
  for(var i = menu.GetItemCount() - 1; i >= 0; i--) {
   var item = menu.GetItem(i);
   var isItemChecked = item.name.substr(0, item.name.indexOf("|")) == this.GetFilterCondition(columnIndex);
   item.SetChecked(isItemChecked);
   if(isItemChecked)
    checkedItemIndex = item.index;
   item.SetVisible(this.GetFilterRowMenuItemVisible(item, column));
  }
  menu.ShowAtElement(element);
  this.filterRowMenuColumnIndex = columnIndex;
  this.currentCheckedItemIndex = checkedItemIndex;
  if(this.GetEnableAccessibility())
   menu.accessibleFocusElement = element;
 },
 FilterRowMenuItemClick: function(item) { 
  var itemName = item.name.substr(0, item.name.indexOf("|"));
  item.menu.ProcessLostFocus();
  if(this.currentCheckedItemIndex === item.index)
   return;
  if(this.IsMultiColumnAutoFilter() || this.IsCustDialogVisible()) {
   this.SetFilterCondition(this.filterRowMenuColumnIndex, parseInt(itemName));
   this.OnChange(this.filterRowMenuColumnIndex);
  } else {
   var args = [this.filterRowMenuColumnIndex, itemName];
   if(!this.IsCustDialogVisible())
    this.SendCallbackToChangeFilterCondition(args);
  }
 },
 GetFilterRowMenuItemVisible: function(item, column) {
  if(column.filterRowTypeKind) {
   var visible = item.name.indexOf(column.filterRowTypeKind) > -1;
   if(!visible && column.showFilterMenuLikeItem)
    return item.name.indexOf("L") > -1;
   return visible;
  }
  return false;
 },
 SetHeaderFilterPopupHtml: function(htmlObj, columnIndex) {
  var processedByCustDialogHelper = false;
  if(this.IsCustDialogVisible())
   processedByCustDialogHelper = this.GetCustDialogHelper().ProcessHeaderFilterCallback(htmlObj, columnIndex);
  if(!processedByCustDialogHelper)
   ASPx.GridFilterHelper.prototype.SetHeaderFilterPopupHtml.call(this, htmlObj, columnIndex);
 },
 OnListBoxSelectionChanged: function(listBox) {
  ASPx.GridFilterHelper.prototype.OnListBoxSelectionChanged.call(this, listBox);
  this.OnChange(listBox.gvColumn.index);
 },
 OnSelectAllCheckedChanged: function(checkBox) {
  ASPx.GridFilterHelper.prototype.OnSelectAllCheckedChanged.call(this, checkBox);
  this.OnChange(checkBox.gvColumn.index);
 },
 OnDateRangePickerValueChanged: function(editor) {
  ASPx.GridFilterHelper.prototype.OnDateRangePickerValueChanged.call(this, editor);
  this.OnChange(editor.gvColumn.index);
 },
 OnCalendarSelectionChanged: function(editor) {
  ASPx.GridFilterHelper.prototype.OnCalendarSelectionChanged.call(this, editor);
  this.OnChange(editor.gvColumn.index);
 },
 RequireApplyHeaderFilterNow: function(mode, columnIndex) {
  if(this.IsCustDialogVisible())
   return false;
  return ASPx.GridFilterHelper.prototype.RequireApplyHeaderFilterNow.call(this, mode, columnIndex);
 },
 OnControlsInitialized: function() {
  ASPx.GridFilterHelper.prototype.OnControlsInitialized.call(this);
  if(this.IsCustDialogVisible())
   this.GetCustDialogHelper().ClearHeaderFilterAfterLoadValues();
 },
 AdjustHeaderFilterListBox: function(column) {
  if(this.IsCustDialogVisible() && this.RequireShowHeaderFilterListBoxScrollbar(column)) {
   var listBox = this.GetListBox(column.index);
   var listBoxMainElement = listBox && listBox.GetMainElement();
   if(listBoxMainElement) {
    ASPx.SetStyles(listBoxMainElement, { minHeight: listBoxMainElement.parentNode.offsetHeight });
    listBox.AdjustControl();
   }
  } else
   ASPx.GridFilterHelper.prototype.AdjustHeaderFilterListBox.call(this, column);
 }
});
ASPx.GridViewFilterHelper = GridViewFilterHelper;
})();
